@@ -22,17 +22,40 @@ class SubtleNoise(Augmentation):
     ):
         super().__init__(p=p)
         self.range = range
-        self.add_subtle_noise = np.vectorize(
-            lambda x: max(0, min(255, x + random.randint(-self.range, self.range))),
-        )
 
     # Constructs a string representation of this Augmentation.
     def __repr__(self):
         return f"SubtleNoise(range={self.range}, p={self.p})"
 
+    # generate mask of noise and add it to input image
+    def add_subtle_noise(self, image):
+        # get image size
+        ysize, xsize = image.shape[:2]
+
+        # generate 2d mask of random noise
+        image_noise = np.random.randint(-self.range, self.range, size=(ysize, xsize))
+
+        # add noise to image
+        image = image.astype("int") + image_noise
+
+        return image
+
     # Applies the Augmentation to input data.
     def __call__(self, image, layer=None, force=False):
         if force or self.should_run():
             image = image.copy()
-            image = self.add_subtle_noise(image)
-            return image
+
+            # multiple channels image
+            if len(image.shape) > 2:
+                # convert to int to enable negative
+                image = image.astype("int")
+                for i in range(image.shape[2]):
+                    image[:, :, i] = self.add_subtle_noise(image[:, :, i])
+            # single channel image
+            else:
+                image = self.add_subtle_noise(image)
+
+            # clip values between 0-255
+            image = np.clip(image, 0, 255)
+
+            return image.astype("uint8")
